'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _lodash = require('lodash.omit');

var _lodash2 = _interopRequireDefault(_lodash);

var _SelectionUtils = require('./SelectionUtils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }
/**
* Copyright (c) 2015 ZenPayroll
*
* This source code is licensed under the MIT license found in the
* LICENSE file in the root directory of this source tree.
*/

var DEFAULT_TRANSLATIONS = {
  9: /\d/,
  a: /[A-Za-z]/,
  '*': /[A-Za-z0-9]/
};

var BLANK_CHAR = '_';

var MaskedField = function (_React$Component) {
  _inherits(MaskedField, _React$Component);

  function MaskedField(props) {
    _classCallCheck(this, MaskedField);

    var _this = _possibleConstructorReturn(this, (MaskedField.__proto__ || Object.getPrototypeOf(MaskedField)).call(this, props));

    _this._handleFocus = function (e) {
      setTimeout(function () {
        return _this._setSelection(_this._cursorPos);
      }, 0);

      if (_this.props.onFocus) {
        _this.props.onFocus(e);
      }

      _this.setState({ value: _this._buffer.join('') });
    };

    _this._handleBlur = function (e) {
      if (_this._isBufferEmpty()) {
        _this._setValue('');
      }

      if (_this.props.onBlur) {
        _this.props.onBlur(e);
      }
    };

    _this._handleKeyDown = function (e) {
      if (e.key === 'Backspace' || e.key === 'Delete') {
        var _this$_getSelection = _this._getSelection(),
            start = _this$_getSelection.start,
            end = _this$_getSelection.end;

        if (start === end) {
          start = e.key === 'Delete' ? _this._nextNonMaskIdx(start - 1) : _this._prevNonMaskIdx(start);
          end = _this._nextNonMaskIdx(start);
        }

        var value = void 0;
        var pattern = _this._getPattern(start);
        if (pattern && pattern.test(_this._buffer[end])) {
          value = _this._maskedValue(_this.state.value.substring(end), start);
        } else {
          _this._resetBuffer(start, end);
          value = _this._buffer.join('');
        }

        _this._setValue(value);
        _this._cursorPos = Math.max(start, _this._firstNonMaskIdx);

        e.preventDefault();
      }

      if (_this.props.onKeyDown) {
        _this.props.onKeyDown(e);
      }
    };

    _this._handleChange = function (e) {
      var value = _this._maskedValue(e.target.value);
      _this._setValue(value);
      _this._callOnComplete(value);
    };

    if (!props.mask) {
      return _possibleConstructorReturn(_this);
    }

    _this._buffer = _this._initialBuffer();
    _this._cursorPos = _this._firstNonMaskIdx;

    var propsValue = _this._getPropsValue();
    _this.state = {
      // TODO: Any way we can do this in one pass?
      value: propsValue ? _this._maskedValue(propsValue) : ''
    };
    return _this;
  }

  _createClass(MaskedField, [{
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      if (this._cursorPos !== undefined) {
        this._setSelection(this._cursorPos);
      }
    }
  }, {
    key: 'componentDidMount',
    value: function componentDidMount() {
      this._isMounted = true;
      var propsValue = this._getPropsValue();
      if (this.props.mask && typeof propsValue === 'string' && this.state.value !== propsValue) {
        this._callOnChange(this.state.value);
      }
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this._isMounted = false;
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      var props = (0, _lodash2.default)(this.props, 'mask', 'translations', 'onComplete');
      var maskProps = {};
      if (this.props.mask) {
        maskProps = {
          onChange: this._handleChange,
          onKeyDown: this._handleKeyDown,
          onFocus: this._handleFocus,
          onBlur: this._handleBlur,
          value: this.state.value
        };
        props = (0, _lodash2.default)(props, 'valueLink');

        if (!this.props.placeholder) {
          maskProps.placeholder = this._initialBuffer().join('');
        }
      }

      return _react2.default.createElement('input', _extends({ ref: function ref(c) {
          return _this2._input = c;
        } }, props, maskProps, { type: 'text' }));
    }
  }, {
    key: '_getSelection',
    value: function _getSelection() {
      if (this._isMounted) {
        return (0, _SelectionUtils.getSelection)(this._input);
      } else {
        var cursorPos = (this._getPropsValue() || '').length;
        return { start: cursorPos, end: cursorPos };
      }
    }
  }, {
    key: '_setSelection',
    value: function _setSelection(start) {
      var end = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : start;

      if (this._input === document.activeElement) {
        (0, _SelectionUtils.setSelection)(this._input, start, end);
      }
    }
  }, {
    key: '_getPropsValue',
    value: function _getPropsValue() {
      if (this.props.valueLink) {
        return this.props.valueLink.value;
      } else {
        return this.props.value;
      }
    }
  }, {
    key: '_getPattern',
    value: function _getPattern(idx) {
      var maskChar = this.props.mask[idx];
      var pattern = this.props.translations ? this.props.translations[maskChar] : null;

      return pattern || DEFAULT_TRANSLATIONS[maskChar];
    }
  }, {
    key: '_resetBuffer',
    value: function _resetBuffer(start, end) {
      for (var i = start; i < end; ++i) {
        if (this._getPattern(i)) {
          this._buffer[i] = BLANK_CHAR;
        }
      }
    }
  }, {
    key: '_initialBuffer',
    value: function _initialBuffer() {
      var buffer = [];
      for (var idx = 0; idx < this.props.mask.length; ++idx) {
        if (this._getPattern(idx)) {
          if (this._firstNonMaskIdx === undefined) {
            this._firstNonMaskIdx = idx;
          }
          buffer.push('_');
        } else {
          buffer.push(this.props.mask[idx]);
        }
      }

      return buffer;
    }
  }, {
    key: '_isBufferEmpty',
    value: function _isBufferEmpty() {
      var _this3 = this;

      return this._buffer.every(function (char, idx) {
        return !_this3._getPattern(idx) || char === BLANK_CHAR;
      });
    }
  }, {
    key: '_isBufferFull',
    value: function _isBufferFull() {
      var _this4 = this;

      return this._buffer.every(function (char, idx) {
        return !_this4._getPattern(idx) || char !== BLANK_CHAR;
      });
    }
  }, {
    key: '_nextNonMaskIdx',
    value: function _nextNonMaskIdx(idx) {
      var next = idx + 1;
      for (; next < this.props.mask.length; ++next) {
        if (this._getPattern(next)) {
          break;
        }
      }

      return next;
    }
  }, {
    key: '_prevNonMaskIdx',
    value: function _prevNonMaskIdx(idx) {
      var prev = idx - 1;
      for (; prev >= 0; --prev) {
        if (this._getPattern(prev)) {
          break;
        }
      }

      return prev;
    }
  }, {
    key: '_callOnChange',
    value: function _callOnChange(value) {
      if (this.props.valueLink) {
        this.props.valueLink.requestChange(value);
      } else if (this.props.onChange) {
        this.props.onChange({ target: { value: value } });
      }
    }
  }, {
    key: '_callOnComplete',
    value: function _callOnComplete(value) {
      if (this.props.onComplete && this._isBufferFull()) {
        this.props.onComplete(value);
      }
    }
  }, {
    key: '_setValue',
    value: function _setValue(value) {
      if (value !== this.state.value) {
        this._callOnChange(value);
      }
      this.setState({ value: value });
    }
  }, {
    key: '_maskedValue',
    value: function _maskedValue(value) {
      var start = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

      var originalCursorPos = this._cursorPos = this._getSelection().start;
      for (var bufferIdx = start, valueIdx = 0; bufferIdx < this.props.mask.length; ++bufferIdx) {
        var pattern = this._getPattern(bufferIdx);
        if (pattern) {
          var lastPatternIdx = bufferIdx;
          this._buffer[bufferIdx] = BLANK_CHAR;
          while (valueIdx < value.length && bufferIdx < this.props.mask.length) {
            var c = value[valueIdx++];
            if (c === this._buffer[bufferIdx]) {
              bufferIdx++;
            } else if (pattern.test(c)) {
              while (this._buffer[bufferIdx] !== '_') {
                bufferIdx++;
              }
              this._buffer[bufferIdx] = c;
              break;
            } else if (this._cursorPos > lastPatternIdx) {
              this._cursorPos--;
            }
          }

          if (valueIdx >= value.length) {
            this._resetBuffer(lastPatternIdx + 1, this.props.mask.length);
            break;
          }
        } else if (this._buffer[bufferIdx] === value[valueIdx]) {
          if (valueIdx === originalCursorPos) {
            this._cursorPos++;
          }

          valueIdx++;
        } else if (valueIdx <= originalCursorPos) {
          this._cursorPos++;
        }
      }

      return this._buffer.join('');
    }
  }]);

  return MaskedField;
}(_react2.default.Component);

MaskedField.propTypes = {
  mask: _propTypes2.default.string,
  translations: _propTypes2.default.object,
  value: _propTypes2.default.string,
  placeholder: _propTypes2.default.string,
  onChange: _propTypes2.default.func,
  onKeyDown: _propTypes2.default.func,
  onComplete: _propTypes2.default.func,
  onFocus: _propTypes2.default.func,
  onBlur: _propTypes2.default.func,
  // TODO: shape
  valueLink: _propTypes2.default.object
};


module.exports = MaskedField;